@@ -92,6 +92,10 @@ func (cmd MergeCmd) Exec(ctx context.Context, commandStr string, args []string,
 		return 1
 	}
 
+	if dEnv.IsLocked() {
+		return HandleVErrAndExitCode(errhand.VerboseErrorFromError(env.ErrActiveServerLock.New(dEnv.LockFile())), help)
+	}
+
 	var verr errhand.VerboseError
 	if apr.Contains(cli.AbortParam) {
 		mergeActive, err := dEnv.IsMergeActive(ctx)
@@ -198,15 +202,11 @@ func (cmd MergeCmd) Exec(ctx context.Context, commandStr string, args []string,
 			}
 
 			if mergeErr != nil {
-				var verr errhand.VerboseError
-				switch mergeErr {
-				case doltdb.ErrIsAhead:
-					verr = nil
-				default:
-					verr = errhand.VerboseErrorFromError(mergeErr)
-					cli.Println("Unable to stage changes: add and commit to finish merge")
+				if mergeErr == doltdb.ErrIsAhead {
+					return 0
 				}
-				return handleCommitErr(ctx, dEnv, verr, usage)
+				verr := errhand.BuildDError("Merge aborted due to error").AddCause(mergeErr).Build()
+				return HandleVErrAndExitCode(verr, usage)
 			}
 		}
 	}
@@ -239,7 +239,7 @@ func getUnmergedTableCount(ctx context.Context, root *doltdb.RootValue) (int, er
 }
 
 func getCommitMessage(ctx context.Context, apr *argparser.ArgParseResults, dEnv *env.DoltEnv, spec *merge.MergeSpec) (string, errhand.VerboseError) {
-	if m, ok := apr.GetValue(cli.CommitMessageArg); ok {
+	if m, ok := apr.GetValue(cli.MessageArg); ok {
 		return m, nil
 	}
 
