@@ -121,7 +121,13 @@ func GetTableDeltas(ctx context.Context, fromRoot, toRoot *doltdb.RootValue) (de
 		return nil, err
 	}
 
-	return matchTableDeltas(fromDeltas, toDeltas)
+	deltas = matchTableDeltas(fromDeltas, toDeltas)
+	deltas, err = filterUnmodifiedTableDeltas(deltas)
+	if err != nil {
+		return nil, err
+	}
+
+	return deltas, nil
 }
 
 func getFkParentSchs(ctx context.Context, root *doltdb.RootValue, fks ...doltdb.ForeignKey) (map[string]schema.Schema, error) {
@@ -143,7 +149,31 @@ func getFkParentSchs(ctx context.Context, root *doltdb.RootValue, fks ...doltdb.
 	return schs, nil
 }
 
-func matchTableDeltas(fromDeltas, toDeltas []TableDelta) (deltas []TableDelta, err error) {
+func filterUnmodifiedTableDeltas(deltas []TableDelta) ([]TableDelta, error) {
+	var filtered []TableDelta
+	for _, d := range deltas {
+		if d.ToTable == nil || d.FromTable == nil {
+			// Table was added or dropped
+			filtered = append(filtered, d)
+			continue
+		}
+
+		hasChanges, err := d.HasChanges()
+		if err != nil {
+			return nil, err
+		}
+
+		if hasChanges {
+			// Take only modified tables
+			filtered = append(filtered, d)
+		}
+	}
+
+	return filtered, nil
+}
+
+func matchTableDeltas(fromDeltas, toDeltas []TableDelta) (deltas []TableDelta) {
+	var matchedNames []string
 	from := make(map[string]TableDelta, len(fromDeltas))
 	for _, f := range fromDeltas {
 		from[f.FromName] = f
@@ -152,6 +182,9 @@ func matchTableDeltas(fromDeltas, toDeltas []TableDelta) (deltas []TableDelta, e
 	to := make(map[string]TableDelta, len(toDeltas))
 	for _, t := range toDeltas {
 		to[t.ToName] = t
+		if _, ok := from[t.ToName]; ok {
+			matchedNames = append(matchedNames, t.ToName)
+		}
 	}
 
 	match := func(t, f TableDelta) TableDelta {
@@ -170,37 +203,22 @@ func matchTableDeltas(fromDeltas, toDeltas []TableDelta) (deltas []TableDelta, e
 	}
 
 	deltas = make([]TableDelta, 0)
-	for _, f := range from {
-		var matched TableDelta
 
-		// optimistically look for a match by name
-		t, ok := to[f.FromName]
-		if ok && schemasOverlap(t.ToSch, f.FromSch) {
-			matched = match(t, f)
+	for _, name := range matchedNames {
+		t := to[name]
+		f := from[name]
+		if schemasOverlap(t.ToSch, f.FromSch) {
+			matched := match(t, f)
+			deltas = append(deltas, matched)
 			delete(from, f.FromName)
 			delete(to, t.ToName)
 		}
+	}
 
-		if !ok {
-			// otherwise, search pairwise
-			for _, t = range to {
-				if schemasOverlap(f.FromSch, t.ToSch) {
-					matched = match(t, f)
-					delete(from, f.FromName)
-					delete(to, t.ToName)
-					break
-				}
-			}
-		}
-
-		if matched.ToTable != nil && matched.FromTable != nil {
-			hasChanges, err := matched.HasChanges()
-			if err != nil {
-				return nil, err
-			}
-
-			// See if matched is worth appending
-			if hasChanges {
+	for _, f := range from {
+		for _, t := range to {
+			if schemasOverlap(f.FromSch, t.ToSch) {
+				matched := match(t, f)
 				deltas = append(deltas, matched)
 				delete(from, f.FromName)
 				delete(to, t.ToName)
@@ -216,7 +234,7 @@ func matchTableDeltas(fromDeltas, toDeltas []TableDelta) (deltas []TableDelta, e
 		deltas = append(deltas, t)
 	}
 
-	return deltas, nil
+	return deltas
 }
 
 func schemasOverlap(from, to schema.Schema) bool {
