@@ -29,7 +29,6 @@ import (
 	"github.com/dolthub/dolt/go/libraries/doltcore/schema"
 	"github.com/dolthub/dolt/go/libraries/doltcore/schema/typeinfo"
 	"github.com/dolthub/dolt/go/store/hash"
-	"github.com/dolthub/dolt/go/store/prolly/shim"
 	"github.com/dolthub/dolt/go/store/prolly/tree"
 	"github.com/dolthub/dolt/go/store/types"
 )
@@ -44,7 +43,7 @@ const (
 	// IndexNameRegexStr is the regular expression that valid indexes must match.
 	// From the unquoted identifiers: https://dev.mysql.com/doc/refman/8.0/en/identifiers.html
 	// We also allow the '-' character from quoted identifiers.
-	IndexNameRegexStr = `^[-$_0-9a-zA-Z]+$`
+	IndexNameRegexStr = "^[-`$_0-9a-zA-Z]+$"
 )
 
 var (
@@ -79,8 +78,8 @@ type Table struct {
 }
 
 // NewNomsTable creates a noms Struct which stores row data, index data, and schema.
-func NewNomsTable(ctx context.Context, vrw types.ValueReadWriter, sch schema.Schema, rows types.Map, indexes durable.IndexSet, autoIncVal types.Value) (*Table, error) {
-	dt, err := durable.NewNomsTable(ctx, vrw, sch, rows, indexes, autoIncVal)
+func NewNomsTable(ctx context.Context, vrw types.ValueReadWriter, ns tree.NodeStore, sch schema.Schema, rows types.Map, indexes durable.IndexSet, autoIncVal types.Value) (*Table, error) {
+	dt, err := durable.NewNomsTable(ctx, vrw, ns, sch, rows, indexes, autoIncVal)
 	if err != nil {
 		return nil, err
 	}
@@ -89,12 +88,28 @@ func NewNomsTable(ctx context.Context, vrw types.ValueReadWriter, sch schema.Sch
 }
 
 // NewTable creates a durable object which stores row data, index data, and schema.
-func NewTable(ctx context.Context, vrw types.ValueReadWriter, sch schema.Schema, rows durable.Index, indexes durable.IndexSet, autoIncVal types.Value) (*Table, error) {
-	dt, err := durable.NewTable(ctx, vrw, sch, rows, indexes, autoIncVal)
+func NewTable(ctx context.Context, vrw types.ValueReadWriter, ns tree.NodeStore, sch schema.Schema, rows durable.Index, indexes durable.IndexSet, autoIncVal types.Value) (*Table, error) {
+	dt, err := durable.NewTable(ctx, vrw, ns, sch, rows, indexes, autoIncVal)
+	if err != nil {
+		return nil, err
+	}
+	return &Table{table: dt}, nil
+}
+
+func NewEmptyTable(ctx context.Context, vrw types.ValueReadWriter, ns tree.NodeStore, sch schema.Schema) (*Table, error) {
+	rows, err := durable.NewEmptyIndex(ctx, vrw, ns, sch)
+	if err != nil {
+		return nil, err
+	}
+	indexes, err := durable.NewIndexSetWithEmptyIndexes(ctx, vrw, ns, sch)
 	if err != nil {
 		return nil, err
 	}
 
+	dt, err := durable.NewTable(ctx, vrw, ns, sch, rows, indexes, nil)
+	if err != nil {
+		return nil, err
+	}
 	return &Table{table: dt}, nil
 }
 
@@ -108,12 +123,8 @@ func (t *Table) ValueReadWriter() types.ValueReadWriter {
 	return durable.VrwFromTable(t.table)
 }
 
-// NodeStore returns the NodeStore for this table.
 func (t *Table) NodeStore() tree.NodeStore {
-	if t == nil {
-		return nil
-	}
-	return tree.NewNodeStore(shim.ChunkStoreFromVRW(t.ValueReadWriter()))
+	return durable.NodeStoreFromTable(t.table)
 }
 
 // SetConflicts sets the merge conflicts for this table.
@@ -279,20 +290,34 @@ func (t *Table) getProllyConflictSchemas(ctx context.Context, tblName string) (b
 		return nil, nil, nil, err
 	}
 
-	baseTbl, err := tableFromRootIsh(ctx, t.ValueReadWriter(), art.Metadata.BaseRootIsh, tblName)
+	baseTbl, baseOk, err := tableFromRootIsh(ctx, t.ValueReadWriter(), t.NodeStore(), art.Metadata.BaseRootIsh, tblName)
 	if err != nil {
 		return nil, nil, nil, err
 	}
-	theirTbl, err := tableFromRootIsh(ctx, t.ValueReadWriter(), art.TheirRootIsh, tblName)
+	theirTbl, theirOK, err := tableFromRootIsh(ctx, t.ValueReadWriter(), t.NodeStore(), art.TheirRootIsh, tblName)
+	if err != nil {
+		return nil, nil, nil, err
+	}
+	if !theirOK {
+		return nil, nil, nil, fmt.Errorf("could not find tbl %s in right root value", tblName)
+	}
+
+	theirSch, err := theirTbl.GetSchema(ctx)
 	if err != nil {
 		return nil, nil, nil, err
 	}
 
-	baseSch, err := baseTbl.GetSchema(ctx)
-	if err != nil {
-		return nil, nil, nil, err
+	// If the table does not exist in the ancestor, pretend it existed and that
+	// it was completely empty.
+	if !baseOk {
+		if schema.SchemasAreEqual(ourSch, theirSch) {
+			return ourSch, ourSch, theirSch, nil
+		} else {
+			return nil, nil, nil, fmt.Errorf("expected our schema to equal their schema since the table did not exist in the ancestor")
+		}
 	}
-	theirSch, err := theirTbl.GetSchema(ctx)
+
+	baseSch, err := baseTbl.GetSchema(ctx)
 	if err != nil {
 		return nil, nil, nil, err
 	}
@@ -300,19 +325,16 @@ func (t *Table) getProllyConflictSchemas(ctx context.Context, tblName string) (b
 	return baseSch, ourSch, theirSch, nil
 }
 
-func tableFromRootIsh(ctx context.Context, vrw types.ValueReadWriter, h hash.Hash, tblName string) (*Table, error) {
-	rv, err := LoadRootValueFromRootIshAddr(ctx, vrw, h)
+func tableFromRootIsh(ctx context.Context, vrw types.ValueReadWriter, ns tree.NodeStore, h hash.Hash, tblName string) (*Table, bool, error) {
+	rv, err := LoadRootValueFromRootIshAddr(ctx, vrw, ns, h)
 	if err != nil {
-		return nil, err
+		return nil, false, err
 	}
 	tbl, ok, err := rv.GetTable(ctx, tblName)
 	if err != nil {
-		return nil, err
+		return nil, false, err
 	}
-	if !ok {
-		return nil, fmt.Errorf("could not find tbl %s in root value", tblName)
-	}
-	return tbl, nil
+	return tbl, ok, nil
 }
 
 func (t *Table) getNomsConflictSchemas(ctx context.Context) (base, sch, mergeSch schema.Schema, err error) {
@@ -410,7 +432,7 @@ func (t *Table) HashOf() (hash.Hash, error) {
 // Calls to UpdateNomsRows will not be written to the database.  The root must
 // be updated with the updated table, and the root must be committed or written.
 func (t *Table) UpdateNomsRows(ctx context.Context, updatedRows types.Map) (*Table, error) {
-	table, err := t.table.SetTableRows(ctx, durable.IndexFromNomsMap(updatedRows, t.ValueReadWriter()))
+	table, err := t.table.SetTableRows(ctx, durable.IndexFromNomsMap(updatedRows, t.ValueReadWriter(), t.NodeStore()))
 	if err != nil {
 		return nil, err
 	}
