@@ -45,7 +45,7 @@ var _ TypeInfo = (*blobStringType)(nil)
 
 func CreateBlobStringTypeFromParams(params map[string]string) (TypeInfo, error) {
 	var length int64
-	var collation sql.Collation
+	var collation sql.CollationID
 	var err error
 	if collationStr, ok := params[blobStringTypeParam_Collate]; ok {
 		collation, err = sql.ParseCollation(nil, &collationStr, false)
