@@ -43,7 +43,7 @@ type jsonValueTest struct {
 }
 
 func TestJsonValues(t *testing.T) {
-	SkipByDefaultInCI(t)
+	sqle.SkipByDefaultInCI(t)
 	setupCommon := []testCommand{
 		{cmd.SqlCmd{}, args{"-q", `create table js (pk int primary key, js json);`}},
 	}
@@ -154,7 +154,7 @@ func testJsonValue(t *testing.T, test jsonValueTest, setupCommon []testCommand)
 
 // round-trips large random JSON objects through the SQL engine
 func TestLargeJsonObjects(t *testing.T) {
-	SkipByDefaultInCI(t)
+	sqle.SkipByDefaultInCI(t)
 	setupCommon := []testCommand{
 		{cmd.SqlCmd{}, args{"-q", `create table js (pk int primary key, js json);`}},
 	}
