@@ -22,8 +22,10 @@ import (
 	"math"
 	"testing"
 
+	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
+	"github.com/dolthub/dolt/go/store/hash"
 	"github.com/dolthub/dolt/go/store/prolly/message"
 	"github.com/dolthub/dolt/go/store/val"
 )
@@ -33,10 +35,11 @@ func TestWriteImmutableTree(t *testing.T) {
 		inputSize int
 		chunkSize int
 		err       error
+		checkSum  bool
 	}{
 		{
 			inputSize: 100,
-			chunkSize: 5,
+			chunkSize: 40,
 		},
 		{
 			inputSize: 100,
@@ -48,19 +51,19 @@ func TestWriteImmutableTree(t *testing.T) {
 		},
 		{
 			inputSize: 255,
-			chunkSize: 5,
+			chunkSize: 40,
 		},
 		{
 			inputSize: 243,
-			chunkSize: 5,
+			chunkSize: 40,
 		},
 		{
 			inputSize: 47,
-			chunkSize: 3,
+			chunkSize: 40,
 		},
 		{
 			inputSize: 200,
-			chunkSize: 7,
+			chunkSize: 40,
 		},
 		{
 			inputSize: 200,
@@ -68,12 +71,47 @@ func TestWriteImmutableTree(t *testing.T) {
 		},
 		{
 			inputSize: 1,
-			chunkSize: 5,
+			chunkSize: 40,
 		},
 		{
 			inputSize: 20,
 			chunkSize: 500,
 		},
+		{
+			inputSize: 1_000,
+			chunkSize: 47,
+			checkSum:  false,
+		},
+		{
+			inputSize: 1_000,
+			chunkSize: 53,
+			checkSum:  false,
+		},
+		{
+			inputSize: 1_000,
+			chunkSize: 67,
+			checkSum:  false,
+		},
+		{
+			inputSize: 10_000,
+			chunkSize: 89,
+			checkSum:  false,
+		},
+		{
+			inputSize: 50_000_000,
+			chunkSize: 4000,
+			checkSum:  false,
+		},
+		{
+			inputSize: 50_000_000,
+			chunkSize: 32_000,
+			checkSum:  false,
+		},
+		{
+			inputSize: 50_000_000,
+			chunkSize: 33_000,
+			err:       ErrInvalidChunkSize,
+		},
 		{
 			inputSize: 10,
 			chunkSize: 1,
@@ -84,6 +122,11 @@ func TestWriteImmutableTree(t *testing.T) {
 			chunkSize: -1,
 			err:       ErrInvalidChunkSize,
 		},
+		{
+			inputSize: 10,
+			chunkSize: 39,
+			err:       ErrInvalidChunkSize,
+		},
 	}
 
 	for _, tt := range tests {
@@ -95,7 +138,7 @@ func TestWriteImmutableTree(t *testing.T) {
 			ctx := context.Background()
 			r := bytes.NewReader(buf)
 			ns := NewTestNodeStore()
-			serializer := message.ProllyMapSerializer{Pool: ns.Pool()}
+			serializer := message.NewBlobSerializer(ns.Pool())
 			root, err := buildImmutableTree(ctx, r, ns, serializer, tt.chunkSize)
 			if tt.err != nil {
 				require.True(t, errors.Is(err, tt.err))
@@ -108,63 +151,84 @@ func TestWriteImmutableTree(t *testing.T) {
 			expSum := expectedSum(tt.inputSize)
 			expUnfilled := expectedUnfilled(tt.inputSize, tt.chunkSize)
 
+			intChunkSize := int(math.Ceil(float64(tt.chunkSize) / float64(hash.ByteLen)))
+
 			unfilledCnt := 0
 			sum := 0
 			byteCnt := 0
 			WalkNodes(ctx, root, ns, func(ctx context.Context, n Node) error {
 				var keyCnt int
 				if n.IsLeaf() {
-					byteCnt += len(n.values.Buf)
+					byteCnt += len(n.values.Items)
 					for _, i := range n.getValue(0) {
 						sum += int(i)
 					}
-					keyCnt = len(n.values.Buf)
+					keyCnt = len(n.values.Items)
+					if keyCnt != tt.chunkSize {
+						unfilledCnt += 1
+					}
 				} else {
 					keyCnt = n.Count()
-				}
-				if keyCnt != tt.chunkSize {
-					unfilledCnt += 1
+					if keyCnt < intChunkSize {
+						unfilledCnt += 1
+					}
 				}
 				return nil
 			})
 
-			require.Equal(t, expLevel, root.Level())
-			require.Equal(t, expSum, sum)
-			require.Equal(t, tt.inputSize, byteCnt)
-			require.Equal(t, expUnfilled, unfilledCnt)
-			require.Equal(t, expSubtrees, root.getSubtreeCounts())
+			assert.Equal(t, expLevel, root.Level())
+			if tt.checkSum {
+				assert.Equal(t, expSum, sum)
+			}
+			assert.Equal(t, tt.inputSize, byteCnt)
+			assert.Equal(t, expUnfilled, unfilledCnt)
+			if expLevel > 0 {
+				root = root.loadSubtrees()
+				for i := range expSubtrees {
+					assert.Equal(t, expSubtrees[i], root.getSubtreeCount(i))
+				}
+			}
 		})
 	}
 }
 
 func expectedLevel(size, chunk int) int {
-	l := 0
-	for size > chunk {
-		size = size / chunk
+	if size <= chunk {
+		return 0
+	}
+	size = int(math.Ceil(float64(size) / float64(chunk)))
+	l := 1
+	intChunk := int(math.Ceil(float64(chunk) / float64(hash.ByteLen)))
+	for size > intChunk {
+		size = int(math.Ceil(float64(size) / float64(intChunk)))
 		l += 1
 	}
 	return l
 }
 
-func expectedSubtrees(size, chunk int) SubtreeCounts {
+func expectedSubtrees(size, chunk int) subtreeCounts {
 	if size <= chunk {
-		return SubtreeCounts{0}
+		return subtreeCounts{0}
 	}
+	l := expectedLevel(size, chunk)
+
 	size = int(math.Ceil(float64(size) / float64(chunk)))
-	l := chunk
-	for l < size {
-		l *= chunk
-	}
-	l /= chunk
+	intChunk := int(math.Ceil(float64(chunk) / float64(hash.ByteLen)))
 
-	res := make(SubtreeCounts, 0)
-	for size > l {
-		res = append(res, uint64(l))
-		size -= l
-	}
-	res = append(res, uint64(size))
+	filledSubtree := int(math.Pow(float64(intChunk), float64(l-1)))
 
-	return res
+	subtrees := make(subtreeCounts, 0)
+	for size > filledSubtree {
+		subtrees = append(subtrees, uint64(filledSubtree))
+		size -= filledSubtree
+	}
+	if size > 0 {
+		subtrees = append(subtrees, uint64(size))
+	}
+	if len(subtrees) > intChunk {
+		panic("unreachable")
+	}
+	return subtrees
 }
 
 func expectedSum(size int) int {
@@ -172,30 +236,30 @@ func expectedSum(size int) int {
 }
 
 func expectedUnfilled(size, chunk int) int {
-	l := chunk
-	for l < size {
-		l *= chunk
+	if size == chunk {
+		return 0
+	} else if size < chunk {
+		return 1
 	}
-	l /= chunk
-	size -= l
-	cnt := 0
-	i := 1
-	for size > 0 {
-		if l > size {
-			if i < chunk-1 {
-				cnt += 1
-			}
-			l /= chunk
-			i = 0
-		} else {
-			size -= l
-			i++
+
+	var unfilled int
+	// level 0 is special case
+	if size%chunk != 0 {
+		unfilled += 1
+	}
+	size = int(math.Ceil(float64(size) / float64(chunk)))
+
+	intChunk := int(math.Ceil(float64(chunk) / float64(hash.ByteLen)))
+	for size > intChunk {
+		if size%intChunk != 0 {
+			unfilled += 1
 		}
+		size = int(math.Ceil(float64(size) / float64(intChunk)))
 	}
-	if i < chunk {
-		cnt += 1
+	if size < intChunk {
+		unfilled += 1
 	}
-	return cnt
+	return unfilled
 }
 
 func TestImmutableTreeWalk(t *testing.T) {
@@ -205,33 +269,43 @@ func TestImmutableTreeWalk(t *testing.T) {
 		keyCnt    int
 	}{
 		{
-			blobLen:   25,
-			chunkSize: 6,
+			blobLen:   250,
+			chunkSize: 41,
 			keyCnt:    4,
 		},
 		{
-			blobLen:   25,
-			chunkSize: 5,
+			blobLen:   250,
+			chunkSize: 40,
 			keyCnt:    4,
 		},
 		{
 			blobLen:   378,
-			chunkSize: 5,
+			chunkSize: 43,
 			keyCnt:    12,
 		},
 		{
 			blobLen:   5000,
-			chunkSize: 12,
+			chunkSize: 40,
 			keyCnt:    6,
 		},
 		{
 			blobLen:   1,
-			chunkSize: 12,
+			chunkSize: 40,
 			keyCnt:    6,
 		},
 		{
 			blobLen:   0,
-			chunkSize: 12,
+			chunkSize: 40,
+			keyCnt:    6,
+		},
+		{
+			blobLen:   50_000_000,
+			chunkSize: 4000,
+			keyCnt:    1,
+		},
+		{
+			blobLen:   10_000,
+			chunkSize: 83,
 			keyCnt:    6,
 		},
 	}
@@ -245,21 +319,28 @@ func TestImmutableTreeWalk(t *testing.T) {
 				cnt++
 				return nil
 			})
-			require.Equal(t, leafAddrCnt(tt.blobLen, tt.chunkSize)*tt.keyCnt+1, cnt)
+			require.Equal(t, blobAddrCnt(tt.blobLen, tt.chunkSize)*tt.keyCnt+1, cnt)
 		})
 	}
 }
 
-func leafAddrCnt(size, chunk int) int {
+func blobAddrCnt(size, chunk int) int {
 	if size == 0 {
 		return 0
 	}
-	l := 1
-	for size > chunk {
-		size = int(math.Ceil(float64(size) / float64(chunk)))
-		l += size
+	if size <= chunk {
+		return 1
 	}
-	return l
+	size = int(math.Ceil(float64(size) / float64(chunk)))
+	l := 1
+	sum := size
+	intChunk := int(math.Ceil(float64(chunk) / float64(hash.ByteLen)))
+	for size > intChunk {
+		size = int(math.Ceil(float64(size) / float64(intChunk)))
+		sum += size
+		l += 1
+	}
+	return sum + 1
 }
 
 func newTree(t *testing.T, ns NodeStore, keyCnt, blobLen, chunkSize int) Node {
@@ -280,8 +361,8 @@ func newTree(t *testing.T, ns NodeStore, keyCnt, blobLen, chunkSize int) Node {
 		tuples[i][1] = valBld.Build(sharedPool)
 	}
 
-	serializer := message.ProllyMapSerializer{Pool: ns.Pool(), ValDesc: valDesc}
-	chunker, err := newEmptyChunker(ctx, ns, serializer)
+	s := message.NewProllyMapSerializer(valDesc, ns.Pool())
+	chunker, err := newEmptyChunker(ctx, ns, s)
 	require.NoError(t, err)
 	for _, pair := range tuples {
 		err := chunker.AddPair(ctx, Item(pair[0]), Item(pair[1]))
